import java.io.Serializable;


/**
 * Base class for persistent model objects which have their identifier
 * assigned manually.
*/
public abstract class ManualIdentifier extends PersistentEntity {

	// --------------------------------------------------------- Static Fields

	/** Version for serialization. */
	private static final long serialVersionUID = 1L;

	// --------------------------------------------------------- Public Methods

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public int compareTo(PersistentEntity o) {
		return ((Comparable<Serializable>) getId()).compareTo(o.getId());
	}

    /**
	 * Indicates whether some other object is "equal to" this one. This method
	 * uses the identifier for the test so it can only be used for objects with
	 * a manually assigned identifier.
	 * 
	 * @param o
	 *            the object to compare this instance to
	 * @return <code>true</code> if the identifiers match; <code>false</code>
	 *         otherwise.
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object o) {
		return equalsClass(o)
				&& getId().equals(((ManualIdentifier) o).getId());
	}

	/**
	 * Returns a hash code value for the object. This method uses the identifier
	 * for the calculation. This can only be done for objects with a manually
	 * assigned identifier as the hash code should not change after an object
	 * has been persisted.
	 * 
	 * @return a hash code value for this object.
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return getId() == null ? super.hashCode() : getId().hashCode();
	}

	// ----------------------------------------------------- Protected Methods

	/**
     * Indicates whether some other object is an instance of the same class.
     * 
     * @param o the object's class to compare this instance to
     * @return <code>true</code> if this object's class matches this instance
     * @see #equals(java.lang.Object)
     */
    protected abstract boolean equalsClass(Object o);
}
