
import java.io.Serializable;

/**
 * Base class for persistent model objects.
 */
public abstract class PersistentEntity implements Comparable<PersistentEntity>,
		Serializable {

	// --------------------------------------------------------- Static Fields

	/** Version for serialization. */
	private static final long serialVersionUID = 1L;

	// ------------------------------------------------------- Instance Fields

	/** 
	 * Data base version for this entity. Managed by the O/R mapper. A
	 * negative value indicates an entity that has not been persisted yet.
	 */
	private int dbVersion = -1;

	// -------------------------------------------------------- Public Methods

	/**
	 * Get a label for user interfaces. The default implementation returns
	 * the identifier if it is set or an empty string otherwise.
	 * 
	 * @return a string representation that is suitable for user interfaces
	 *         (never <code>null</code>)
	 */
	public String getLabel() {
		return getId() != null ? getId().toString() : "";
	}

	/**
	 * Get the identifier of this entity. This method is named
	 * <code>getId()</code> because Hibernate proxies won't be initialized
	 * when their ID is retrieved this way. See
	 * http://www.hibernate.org/118.html#A21
	 * 
	 * @return The identifier or <code>null</code> if none is set yet because
	 *         the instance has not been persisted yet.
	 */
	public abstract Serializable getId();

	/**
	 * Tests if this entity is already saved to the data base.
	 * 
	 * @return <code>true</code> if the entity is saved,
	 *         <code>false</code> otherwise
	 */
	public boolean isSaved() {
		return dbVersion >= 0;
	}

	/**
	 * Returns a string representation that briefly describes the entity which
	 * can for example be used in log files or the change log. The default
	 * implementation returns the identifier if its set or an empty string
	 * otherwise.
	 *
	 * @return a string representation that is suitable for user interfaces
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getId() != null ? getId().toString() : "";
	}

	// ----------------------------------- Generated Getter and Setter Methods

	/**
	 * Generated getter method which just returns the property without
	 * additional processing.
	 *
	 * @return the requested property
	 */
	public int getDbVersion() {
		return dbVersion;
	}

	/**
	 * Generated setter method which just sets the property without additional
	 * processing.
	 * 
	 * @param dbVersion
	 *            the property to set
	 */
	public void setDbVersion(int dbVersion) {
		this.dbVersion = dbVersion;
	}
}
